import logging

class kFormatter(logging.Formatter):
    grey = "\x1b[34;20m"
    blue = "\x1b[38;20m"
    yellow = "\x1b[36;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[41;1m"
    reset = "\x1b[0m"
    format = "%(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"


class kColorFormatter(kFormatter):
    #format dictionary
    FORMATS = {
        logging.DEBUG: kFormatter.grey + kFormatter.format + kFormatter.reset,
        logging.INFO: kFormatter.blue + kFormatter.format + kFormatter.reset,
        logging.WARNING: kFormatter.yellow + kFormatter.format + kFormatter.reset,
        logging.ERROR: kFormatter.red + kFormatter.format + kFormatter.reset,
        logging.CRITICAL: kFormatter.bold_red + kFormatter.format + kFormatter.reset
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)

class kNoColorFormatter(kFormatter):
    #format dictionary
    FORMATS = {
        logging.DEBUG: kFormatter.format,
        logging.INFO: kFormatter.format,
        logging.WARNING: kFormatter.format,
        logging.ERROR: kFormatter.format,
        logging.CRITICAL: kFormatter.format
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)




import logging
import inspect
from executing import Source
from .kFormatter import kColorFormatter, kNoColorFormatter

class kLogger:

    def __init__(self, tag, logfile=None, loglevel="DEBUG"):
        if not loglevel:
            loglevel = "DEBUG"
        self.tag = tag 
        self.logfile = logfile
        self.loglevel = loglevel
        self.logger = logging.getLogger(self.tag)
        self.logger.setLevel(self.loglevel.upper())

        if not self.logfile:
            self.ch = logging.StreamHandler()
            self.ch.setFormatter(kColorFormatter())
        else:
            self.ch = logging.FileHandler(self.logfile)
            self.ch.setFormatter(kNoColorFormatter())
        self.ch.setLevel(self.loglevel.upper())

        if not self.logger.handlers:
            self.logger.addHandler(self.ch)

    def __call__(self, *args):
        if args:
            for arg in args:
                callFrame = inspect.currentframe().f_back
                callNode = Source.executing(callFrame).node
                source = Source.for_frame(callFrame)
                expression = source.asttokens().get_text(callNode.args[0])
                self.logger.info(f'{expression} | {arg}', stacklevel=2)
        else:
            self.logger.info("", stacklevel=2)

    def debug(self, message):
        self.logger.debug(message, stacklevel=2)

    def info(self, message):
        self.logger.info(message, stacklevel=2)

    def warning(self, message):
        self.logger.warning(message, stacklevel=2)

    def error(self, message):
        self.logger.error(message, stacklevel=2)

    def critical(self, message):
        self.logger.critical(message, stacklevel=2, stack_info=True)

    def setLevel(self, level):
        self.loglevel = level.upper()
        self.logger.setLevel(level.upper())
        self.ch.setLevel(level.upper())

    def setFile(self, file):
        self.logfile = file
        if file:
            self.logger.handlers.clear()
            self.logger.setLevel(self.loglevel.upper())

            self.ch = logging.FileHandler(self.logfile)
            self.ch.setFormatter(kNoColorFormatter())
            self.ch.setLevel(self.loglevel.upper())
            self.logger.addHandler(self.ch)

    def addFile(self, file):
        ch = logging.FileHandler(file)
        ch.setFormatter(kNoColorFormatter())
        ch.setLevel(self.loglevel.upper())
        self.logger.addHandler(ch)
        
def get_logger(tag, logfile, loglevel):
    return kLogger(tag, logfile, loglevel)



import logging
import argparse
import threading
import multiprocessing as mp
from kLogger import kLogger


def basic_test(log):
    #test default usage
    log.debug("debug message")
    log.info("info message")
    log.warning("warning message")
    log.error("error message")
    log.critical("critical message")

    #test calls
    log()
    x = 10
    log(x)

def test(logfile, loglevel):
    #test creation
    log = kLogger("klogs", logfile, loglevel)
    basic_test(log)

def test_threads(logfile, loglevel):
    log = kLogger("klogs", logfile, loglevel)
    
    def thread():
        basic_test(log)

    info_string = "######## In thread ########"
    if logfile:
        with open(logfile, "a") as fd:
            fd.write(info_string)
            fd.write("\n")
    else:
        print(info_string)
    t = threading.Thread(target=thread)
    t.start()
    t.join()
    info_string = "######## Outside thread ########"
    if logfile:
        with open(logfile, "a") as fd:
            fd.write(info_string)
            fd.write("\n")
    else:
        print(info_string)
    basic_test(log)


def process(logfile, loglevel):
    log = kLogger("klogs", logfile, loglevel)
    log.debug("debug message")
    log.info("info message")
    log.warning("warning message")
    log.error("error message")
    log.critical("critical message")

    #test calls
    log()
    x = 10
    log(x)

def test_process(logfile, loglevel):
    log = kLogger("klogs", logfile, loglevel)
    info_string = "######## In process ########"
    if logfile:
        with open(logfile, "a") as fd:
            fd.write(info_string)
            fd.write("\n")
    else:
        print(info_string)
    p = mp.Process(target=process, args=(logfile, loglevel,))
    p.start()
    p.join()
    info_string = "######## Outside process ########"
    if logfile:
        with open(logfile, "a") as fd:
            fd.write(info_string)
            fd.write("\n")
    else:
        print(info_string)
    basic_test(log)

def test_multifile(logfile, loglevel):
    if not logfile:
        logfile = "log"
    log = kLogger("klogs-multifile", logfile, loglevel)
    log.addFile(logfile+"2")
    log.addFile(logfile+"3")
    log.addFile(logfile+"4")
    basic_test(log)

if __name__ == "__main__":
    #argparsing 
    argparser = argparse.ArgumentParser(description='Klogs')
    argparser.add_argument('-f', '--file', help='Log file')
    argparser.add_argument('-l', '--level', help='Log level')
    args = argparser.parse_args()
    info_string = "######## Testing normal usage ########"
    if args.file:
        with open(args.file, "w") as fd:
            fd.write(info_string)
            fd.write("\n")
    else:
        print(info_string)
    test(args.file, args.level)
    info_string = "######## Testing threaded usage ########"
    if args.file:
        with open(args.file, "a") as fd:
            fd.write(info_string)
            fd.write("\n")
    else:
        print(info_string)
    test_threads(args.file, args.level)
    info_string = "######## Testing process usage ########"
    if args.file:
        with open(args.file, "a") as fd:
            fd.write(info_string)
            fd.write("\n")
    else:
        print(info_string)
    test_process(args.file, args.level)
    #This multifile test is kind of dumb / not useful
    # info_string = "######## Testing multi file ########"
    # if args.file:
    #     with open(args.file, "a") as fd:
    #         fd.write(info_string)
    #         fd.write("\n")
    # else:
    #     print(info_string)
    # test_multifile(args.file, args.level)

